from sigma.pipelines.common import logsource_windows, windows_logsource_mapping
from sigma.pipelines.base import Pipeline
from sigma.processing.transformations import ChangeLogsourceTransformation, RuleFailureTransformation, DetectionItemFailureTransformation, FieldMappingTransformation
from sigma.processing.postprocessing import EmbedQueryTransformation
from sigma.processing.conditions import LogsourceCondition, IncludeFieldCondition, ExcludeFieldCondition, RuleProcessingItemAppliedCondition, RuleProcessingCondition
from sigma.processing.pipeline import ProcessingItem, ProcessingPipeline, QueryPostprocessingItem
from sigma.rule import SigmaRule

# --- Logsource helpers ----------------------------

def ls_win_proc():  
    return LogsourceCondition(category="process_creation", product="windows")

def ls_win_sec_logon():  
    return LogsourceCondition(category="authentication", product="windows", service="security")

def ls_linux_proc(): 
    return LogsourceCondition(category="process_creation", product="linux", service="rsyslog")

def ls_nginx_proxy():
    return LogsourceCondition(category="proxy", product="nginx")

def ls_generic_syslog():
    return LogsourceCondition(category="syslog")  

        
@Pipeline
# Processing pipelines should be defined as functions that return a ProcessingPipeline object.
def predictiq_pipeline() -> ProcessingPipeline:       
    """Pipeline básico con tres pasos:
    1. Cambio de propiedades de logsource
    2. Error genérico para logsources no soportados
    3. Manejo de reglas con funciones agregadas
    """
    return ProcessingPipeline(
        name="Basic SIEM Backend Pipeline",
        priority=10,
        items=[
            # 1. Windows Process Creation
            ProcessingItem(
                identifier="predict_iq_proc_fieldmap",
                transformation=FieldMappingTransformation({
                    "Image":       "process_name",
                    "ProcessId":   "process_id",
                    "CommandLine": "command_line",
                    "ParentImage": "parent_process",
                    "User":        "user",
                    "Hostname":    "hostname",
                }),
                rule_conditions=[ ls_win_proc() ],
            ),
            ProcessingItem(
                identifier="predict_iq_proc_logsource",
                transformation=ChangeLogsourceTransformation(
                    category="process", product="windows"
                ),
                rule_conditions=[ ls_win_proc() ],
            ),
            ProcessingItem(
                identifier="predict_iq_proc_unsupported",
                transformation=DetectionItemFailureTransformation(
                    "Campo no soportado para eventos de proceso (IntegrityLevel, imphash, LogonId)"
                ),
                rule_conditions=[ ls_win_proc() ],
                field_name_conditions=[
                    IncludeFieldCondition(fields=["IntegrityLevel", "imphash", "LogonId"])
                ],
            ),

            # 2. Windows Security Logon
            ProcessingItem(
                identifier="predict_iq_logon_fieldmap",
                transformation=FieldMappingTransformation({
                    "event_id":    "event_id",
                    "user":        "user",
                    "hostname":    "hostname",
                    "ip_address":  "source_ip",
                    "log_level":   "log_level",
                }),
                rule_conditions=[ ls_win_sec_logon() ],
            ),
            ProcessingItem(
                identifier="predict_iq_logon_logsource",
                transformation=ChangeLogsourceTransformation(
                    category="authentication", product="windows", service="security"
                ),
                rule_conditions=[ ls_win_sec_logon() ],
            ),

            # 3. Linux Process via Syslog
            ProcessingItem(
                identifier="predict_iq_linux_proc_fieldmap",
                transformation=FieldMappingTransformation({
                    "program":     "process_name",
                    "message":     "message",
                    "timestamp":   "timestamp",
                    "hostname":    "hostname",
                }),
                rule_conditions=[ ls_linux_proc() ],
            ),
            ProcessingItem(
                identifier="predict_iq_linux_proc_logsource",
                transformation=ChangeLogsourceTransformation(
                    category="process_creation", product="linux", service="rsyslog"
                ),
                rule_conditions=[ ls_linux_proc() ],
            ),

            # 4. Nginx Proxy
            ProcessingItem(
                identifier="predict_iq_nginx_proxy_fieldmap",
                transformation=FieldMappingTransformation({
                    "cs-host":    "url_host",
                    "cs-method":  "http_method",
                    "cs-uri":     "url_path",
                    "sc-status":  "status_code",
                    "src_ip":     "source_ip",
                    "dst_ip":     "destination_ip",
                }),
                rule_conditions=[ ls_nginx_proxy() ],
            ),
            ProcessingItem(
                identifier="predict_iq_nginx_proxy_logsource",
                transformation=ChangeLogsourceTransformation(
                    category="proxy", product="nginx"
                ),
                rule_conditions=[ ls_nginx_proxy() ],
            ),

            # 5. Generic Syslog 
            ProcessingItem(
                identifier="predict_iq_syslog_fieldmap",
                transformation=FieldMappingTransformation({
                    "timestamp": "timestamp",
                    "hostname":  "host",
                    "program":   "process_name",
                    "pri":       "severity",
                    "msg":       "message",
                    "raw":       "raw_log",
                }),
                rule_conditions=[ ls_generic_syslog() ],
            ),
            ProcessingItem(
                identifier="predict_iq_syslog_logsource",
                transformation=ChangeLogsourceTransformation(
                    category="syslog"
                ),
                rule_conditions=[ ls_generic_syslog() ],
            ),

            # 6. Catch-all to unsupported log sources
            ProcessingItem(
                identifier="predict_iq_unsupported_rule",
                transformation=RuleFailureTransformation(
                    "Tipo de log aún no soportado por PredicIQ SIEM"
                ),
                rule_condition_negation=True,
                rule_conditions=[
                    RuleProcessingItemAppliedCondition("predict_iq_proc_logsource"),
                    RuleProcessingItemAppliedCondition("predict_iq_logon_logsource"),
                    RuleProcessingItemAppliedCondition("predict_iq_linux_proc_logsource"),
                    RuleProcessingItemAppliedCondition("predict_iq_nginx_proxy_logsource"),
                    RuleProcessingItemAppliedCondition("predict_iq_syslog_logsource"),
                ],
            ),
        ],
    )


from sigma.rule import SigmaRule
from sigma.conversion.base import TextQueryBackend
from sigma.conversion.state import ConversionState, DeferredQueryExpression
from sigma.processing.pipeline import ProcessingPipeline
from sigma.conditions import ConditionItem, ConditionAND, ConditionOR, ConditionNOT, ConditionFieldEqualsValueExpression
from sigma.types import SigmaCompareExpression, SigmaRegularExpression, SigmaRegularExpressionFlag, SigmaString, SigmaNumber
from sigma.pipelines.predictiq import predictiq_pipeline
import sigma.backends as backends
import re
from typing import ClassVar, Dict, Tuple, List, Any, Optional, Union


class PredictIQBackend(TextQueryBackend):
    """predict-iq backend."""

    # Operator precedence: tuple of Condition{AND,OR,NOT} in order of precedence.
    # The backend generates grouping if required
    name : ClassVar[str] = "predict-iq backend"
    formats : ClassVar[Dict[str, str]] = {
        "default": "Simple log search query mode",
    }
    
    requires_pipeline : ClassVar[bool] = False

    # Built-in pipeline
    backend_processing_pipeline : ClassVar[ProcessingPipeline] = predictiq_pipeline()

    # iIn-expressions
    convert_or_as_in : ClassVar[bool] = True                     # Convert OR as in-expression
    convert_and_as_in : ClassVar[bool] = True                    # Convert AND as in-expression
    in_expressions_allow_wildcards : ClassVar[bool] = True       # Values in list can contain wildcards. If set to False (default) only plain values are converted into in-expressions.

    group_expression : ClassVar[str] = "({expr})"

    or_token : ClassVar[str] = "OR"
    and_token : ClassVar[str] = "AND"
    not_token : ClassVar[str] = "NOT"
    eq_token : ClassVar[str] = "="

    contains_token, starts_token, ends_token = "CONTAINS", "STARTS", "ENDS"

    str_double_quote : ClassVar[str] = '"'

    escape_char : ClassVar[str] = "\\"
    wildcard_multi : ClassVar[str] = "*"
    wildcard_single : ClassVar[str] = "*"

    re_expression : ClassVar[str] = "{field}=/{regex}/i"
    re_escape_char : ClassVar[str] = "\\"
    re_escape : ClassVar[Tuple[str]] = ('"')

    cidr_expression : ClassVar[str] = "{field} = IP({value})"

    compare_op_expression : ClassVar[str] = "{field} {operator} {value}"
    compare_operators : ClassVar[Dict[SigmaCompareExpression.CompareOperators, str]] = {
        SigmaCompareExpression.CompareOperators.LT  : "<",
        SigmaCompareExpression.CompareOperators.LTE : "<=",
        SigmaCompareExpression.CompareOperators.GT  : ">",
        SigmaCompareExpression.CompareOperators.GTE : ">=",
    }

    field_null_expression : ClassVar[str] = "{field} = null"

    field_in_list_expression : ClassVar[str] = "{field} IN [{list}]"
    contains_any_expression : ClassVar[Optional[str]] = "{field} CONTAINS-ANY [{list}]"
    contains_all_expression : ClassVar[Optional[str]] = "{field} CONTAINS-ALL [{list}]"
    startswith_any_expression : ClassVar[Optional[str]] = "{field} STARTS-WITH-ANY [{list}]"
    list_separator : ClassVar[str] = ", "

    # Basic conversion “Field = Value (string)” 
    def convert_condition_field_eq_val_str(self, cond, state):
        v = cond.value.to_plain()
        if v.startswith("*") and v.endswith("*"):
            return f'{cond.field} {self.contains_token} "{v.strip("*")}"'
        if v.endswith("*"):
            return f'{cond.field} {self.starts_token} "{v.strip("*")}"'
        if v.startswith("*"):
            return f'{cond.field} {self.ends_token} "{v.strip("*")}"'
        return f'{cond.field} {self.eq_token} "{v}"'

    # Finalization (simple where())
    def finalize_query_default(self, rule: SigmaRule, query: str, _, __):
        return f"where({query})"



from .predictiq import predictiq_pipeline
# TODO: add all pipelines that should be exposed to the user of your backend in the import statement above.
pipelines = {"predictiq_pipeline": predictiq_pipeline}


from .predictiq import PredictIQBackend
# TODO: add all backend classes that should be exposed to the user of your backend in the import statement above.
backends = {
    "predict_iq": PredictIQBackend
}